/**
 * Title:        GridSim Toolkit
 * Version:      GridSim  1.0, 12/09/2001
 * Description:  GridSim (Grid Simulation) Toolkit for Modeling and Simulation of
 *               Parallel and Distributed Systems such as Clusters and Grids
 * Authors:      Manzur Murshed and Rajkumar Buyya
 *               Monash University, Melbourne, Australia
 * Licence:      GPL - http://www.gnu.org/copyleft/gpl.html
 */

package gridsim;
import java.util.*;
import eduni.simjava.*;

/*--------------------------------------------------------------------------- */
// Classes related to GridSim Resources: GIS, Local Resource Schedulers
/*--------------------------------------------------------------------------- */

/**
 * A Grid Information Service (GIS) that provides resource registration
 * and information services. It acts like a yellow page service.
 */
public class GridInformationService extends GridSim
{
  public GridInformationService(String name, double baud_rate) {
    super(name, baud_rate);
  }

  public void body()
  {
    LinkedList ResourceList = new LinkedList();

    // Process events
    Sim_event ev = new Sim_event();
//    for(sim_get_next(new Sim_from_port(input), ev); ev.get_tag()!=GridSimTags.END_OF_SIMULATION; sim_get_next(new Sim_from_port(input), ev))
    for(sim_get_next(ev); ev.get_tag()!=GridSimTags.END_OF_SIMULATION; sim_get_next( ev))
    {
      switch(ev.get_tag())
      {
        case GridSimTags.REGISTER_RESOURCE: // A resource is requesting to register.
          ResourceList.add((Integer) ev.get_data());
          break;

        case GridSimTags.RESOURCE_LIST: // A Broker is requesting for a list of resources.
//				  System.out.println(Sim_system.clock()+" "+get_name()+" sending resource list"+ ev.get_src() + "dst: "+ev.get_dest());
          // Get ID of Broker that send this event
          int requester_id = ((Integer) ev.get_data()).intValue();
          Send(output, GridSimTags.SCHEDULE_NOW, ev.get_tag(),
                        new IO_data(ResourceList, 4*ResourceList.size(), requester_id));
          break;
      }
    }
    TerminateInputOutputEntities();
  }
}
