/**
 * Title:        GridSim Toolkit
 * Version:      GridSim  1.0, 12/09/2001
 * Description:  GridSim (Grid Simulation) Toolkit for Modeling and Simulation of
 *               Parallel and Distributed Systems such as Clusters and Grids
 * @author       R. Buyya and M. Murshed (by surname)
 *               Monash University, Melbourne, Australia
 * Licence:      GPL - http://www.gnu.org/copyleft/gpl.html
 */

package gridsim;

import java.util.*;
import java.io.*;
import eduni.simjava.*;
import gridsim.*;

/**
 * This class wraps up the simulation by signaling the END to the entities.
 * It sends END_OF_SIMULATION message to various GridSim entities on
 * receiving END_OF_SIMULATION from all User entities.
 */

class GridSimShutdown extends GridSim
{
  private int no_of_users;
  private String ReportWriterName;

  public GridSimShutdown(String name, int no_of_users, String ReportWriterName)
  {
    super(name, GridSimTags.DEFAULF_BAUD_RATE);
    this.no_of_users = no_of_users;
    this.ReportWriterName = ReportWriterName;
  }

  /**
   * Discover resources and Send END_OF_SIMULATION message to them.
   */
  public void ShutdownResources()
  {
    LinkedList ResIDList;

    // Requesting the GridInformationService to send a list of resources
    ResIDList = GetGridResourceList();

    // Send END_OF_SIMULATION event to all Resources
    for( int i=0; i < ResIDList.size(); i++)
    {
      int res_id = ((Integer) ResIDList.get(i)).intValue();  // Get Resource ID
      Send(res_id, GridSimTags.SCHEDULE_NOW, GridSimTags.END_OF_SIMULATION);
    }
  }

  public void body()
  {
    Sim_event ev = new Sim_event();

    // wait for shutdown message from all users.
    for(int i=0; i<no_of_users; i++)
      sim_get_next(ev);

    // Invoke report Writter and shutdown
    if(ReportWriterName != null)
    {
      int rep_writer_id = GridSim.GetEntityId(ReportWriterName);
      if(rep_writer_id != -1)
        Send(rep_writer_id, GridSimTags.SCHEDULE_NOW, GridSimTags.END_OF_SIMULATION);
      else
        System.out.println(get_name()+": User defined Report Writer entity "+ReportWriterName+" does not exist!");
    }
    else
      ShutdownGridStatisticsEntity();

    ShutdownResources();
    Send(GridSim.GridInformationServiceEntityId(), GridSimTags.SCHEDULE_NOW, GridSimTags.END_OF_SIMULATION);

    TerminateInputOutputEntities();
//    System.out.println("Shutdown Terminated");
  }
}
