/**
 * Title:        GridSim Toolkit
 * Version:      GridSim  1.0, 12/09/2001
 * Description:  GridSim (Grid Simulation) Toolkit for Modeling and Simulation of
 *               Parallel and Distributed Systems such as Clusters and Grids
 * Authors:      Manzur Murshed and Rajkumar Buyya
 *               Monash University, Melbourne, Australia
 * Licence:      GPL - http://www.gnu.org/copyleft/gpl.html
 */

package gridsim;

/**
 * A class representing Standard PE (Processing Element) rating in
 * MIPS (Million Instructions Per Second). The users can
 * define MIPS rating fora Standard PE.
 * They can use this to define the capability resources or the
 * execution time of Gridlets with respect to Standard PE.
 */
public class GridSimStandardPE
{
  // MIPS = Million Instructions Per Second
  public static int MIPSRating = 100;  // default value

  /** Set Standard PE MIPS Rating
   *  @param MIPSRating_value The value of standard PE MIPS rating
   */
  public static void SetRating(int MIPSRating_value)
  {
    MIPSRating = MIPSRating_value;
  }

  /** Get Standard PE MIPS Rating
   *  @return The value of standard PE MIPS rating
   */
  public static int GetRating()
  {
    return MIPSRating;
  }

  /** Convert Execution Time in Second processor to MIs
   *  @param TimeInSec The execution time in second w.r.t. standard time
   *  @return Equivalent Million Instructions (MIs) for TimeInSec
   */
  public static double toMIs(double TimeInSec)
  {
    return MIPSRating*TimeInSec;
  }
}
