/**
 * Title:        GridSim Toolkit
 * Version:      GridSim  1.0, 12/09/2001
 * Description:  GridSim (Grid Simulation) Toolkit for Modeling and Simulation of
 *               Parallel and Distributed Systems such as Clusters and Grids
 * Authors:      Manzur Murshed and Rajkumar Buyya
 *               Monash University, Melbourne, Australia
 * Licence:      GPL - http://www.gnu.org/copyleft/gpl.html
 */

package gridsim;
import java.util.*;
import java.io.*;
import eduni.simjava.*;
import gridsim.GridSimTags;

/**
 * GridSim Statistics entity.
 */
public class GridStatistics extends GridSim
{
  private boolean active;
  private PrintWriter outFile;
  private LinkedList stats;
  private LinkedList stats_sort_by_cnd;
  private LinkedList stats_sort_by_cd;
  private String[] exclude_from_file;
  private String[] exclude_from_processing;

  public GridStatistics(String name)
  {
    super(name);
    active = false;
    outFile = null;
  }
  public GridStatistics(String name, String filename, boolean append,
          String[] exclude_from_file, String[] exclude_from_processing)
  {
    super(name);
    active = true;

    try {
      FileWriter fw = new FileWriter(filename, append);
      BufferedWriter bw = new BufferedWriter(fw);
      this.outFile = new PrintWriter(bw);
      this.stats = new LinkedList();
      this.exclude_from_file = exclude_from_file;
      this.exclude_from_processing = exclude_from_processing;
    }
    catch(IOException e) {
      System.out.println(get_name()+"-Unable to open statistics file: "+filename);
    }
  }

  /**
   * This class can be used to sort a LinkedList of Stat in ascending order
   * by the field category_name_data.
   */
  class Order_by_category_name_data implements Comparator
  {
    public Order_by_category_name_data() {
      super();
    }
    public int compare(Object a, Object b)
    {
      String c1 = ((Stat) a).get_category();
      String c2 = ((Stat) b).get_category();
      String n1 = ((Stat) a).get_name();
      String n2 = ((Stat) b).get_name();
      String d1 = ((Stat) a).get_data();
      String d2 = ((Stat) b).get_data();

      if(c1.equals(c2))
      {
        if(n1.equals(n2))
          return d1.compareTo(d2);
        else
          return n1.compareTo(n2);
      }
      else
        return c1.compareTo(c2);
    }
  }

  /**
   * This class can be used to sort a LinkedList of Stat in ascending order
   * by the field category_field_data.
   */
  class Order_by_category_data implements Comparator
  {
    public Order_by_category_data()
    {
      super();
    }
    public int compare(Object a, Object b)
    {
      String c1 = ((Stat) a).get_category();
      String c2 = ((Stat) b).get_category();
      String d1 = ((Stat) a).get_data();
      String d2 = ((Stat) b).get_data();

      if(c1.equals(c2))
        return d1.compareTo(d2);
      else
        return c1.compareTo(c2);
    }
  }

  /**
   * This class can be used to sort a LinkedList of Stat in ascending order
   * by the field category_name_field.
   */
  class Order_by_category_name implements Comparator
  {
    public Order_by_category_name()
    {
      super();
    }
    public int compare(Object a, Object b)
    {
      String c1 = ((Stat) a).get_category();
      String c2 = ((Stat) b).get_category();
      String n1 = ((Stat) a).get_name();
      String n2 = ((Stat) b).get_name();

      if(c1.equals(c2))
        return n1.compareTo(n2);
      else
        return c1.compareTo(c2);
    }
  }

  /**
   * This class can be used to sort a LinkedList of Stat in ascending order
   * by the field category_field.
   */
  class Order_by_category implements Comparator
  {
    public Order_by_category()
    {
      super();
    }
    public int compare(Object a, Object b)
    {
      String c1 = ((Stat) a).get_category();
      String c2 = ((Stat) b).get_category();

      return c1.compareTo(c2);
    }
  }

/*  public int count(String category, String data)
  {
    LinkedList stats = stats_sort_by_cd;
    Stat s;
    int cnt = 0;
    int pos = Collections.binarySearch(stats, new Stat(0.0, category, "", data),
            new Order_by_category_data());

    if(pos<0)
      return cnt;

    cnt++;
    for(int i=pos+1; i<stats.size(); i++) {
      s = (Stat) stats.get(i);
      if(s.get_category().equals(category) && s.get_data().equals(data))
        cnt++;
      else
        break;
    }
    for(int i=pos-1; i>=0; i--) {
      s = (Stat) stats.get(i);
      if(s.get_category().equals(category) && s.get_data().equals(data))
        cnt++;
      else
        break;
    }
    return cnt;
  }

  public double add_period(String Category_from, String Category_to,
                           String name_prefix, int no_of_name)
  {
    LinkedList stats = stats_sort_by_cnd;
    double sum = 0.0;
    for(int i=0; i<no_of_name; i++)
    {
      int pos = Collections.binarySearch(stats, new Stat(0.0, Category_to,
                name_prefix+i, ""), new Order_by_category_name());
      Stat s = (Stat) stats.get(pos);
      sum += new Double(s.get_data()).doubleValue();
      pos = Collections.binarySearch(stats, new Stat(0.0, Category_from,
                name_prefix+i, ""), new Order_by_category_name());
      s = (Stat) stats.get(pos);
      sum -= new Double(s.get_data()).doubleValue();
    }
    return sum;
  }
*/

  public Accumulator accumulate(String category)
  {
    LinkedList stats = stats_sort_by_cd;
    Stat s;
    Accumulator acc = new Accumulator();

    int pos = Collections.binarySearch(stats, new Stat(0.0, category, "", ""),
            new Order_by_category());

    if(pos<0)
      return acc;

    s = (Stat) stats.get(pos);
    acc.add(new Double(s.get_data()).doubleValue());

    for(int i=pos+1; i<stats.size(); i++)
    {
      s = (Stat) stats.get(i);
      if(s.get_category().equals(category))
        acc.add(new Double(s.get_data()).doubleValue());
      else
        break;
    }
    for(int i=pos-1; i>=0; i--)
    {
      s = (Stat) stats.get(i);
      if(s.get_category().equals(category))
        acc.add(new Double(s.get_data()).doubleValue());
      else
        break;
    }
    return acc;
  }

  class Times {
    int index;
    int val;

    public Times(int index, int val) {
      this.index = index;
      this.val = val;
    }
  }

  public Accumulator accumulate(String category, String counter)
  {
    LinkedList stats = stats_sort_by_cd;
    Stat s;
    Accumulator acc = new Accumulator();

    class Order_by_index implements Comparator
    {
      public Order_by_index()
      {
        super();
      }
      public int compare(Object a, Object b)
      {
        Integer i1 = new Integer(((Times) a).index);
        Integer i2 = new Integer(((Times) b).index);

        return i1.compareTo(i2);
      }
    }

    LinkedList times = new LinkedList();
    int j, k;

    int pos = Collections.binarySearch(stats, new Stat(0.0, counter, "", ""),
            new Order_by_category());

    if(pos<0)
      return acc;

    s = (Stat) stats.get(pos);
    j = new Integer(s.get_name().substring(s.get_name().indexOf('_')+1)).intValue();
    times.add(new Times(j, new Integer(s.get_data()).intValue()));

    for(int i=pos+1; i<stats.size(); i++)
    {
      s = (Stat) stats.get(i);
      if(s.get_category().equals(counter))
      {
        j = new Integer(s.get_name().substring(s.get_name().indexOf('_')+1)).intValue();
        times.add(new Times(j, new Integer(s.get_data()).intValue()));
      }
      else
        break;
    }
    for(int i=pos-1; i>=0; i--)
    {
      s = (Stat) stats.get(i);
      if(s.get_category().equals(counter))
      {
        j = new Integer(s.get_name().substring(s.get_name().indexOf('_')+1)).intValue();
        times.add(new Times(j, new Integer(s.get_data()).intValue()));
      }
      else
        break;
    }

    Collections.sort(times, new Order_by_index());

    pos = Collections.binarySearch(stats, new Stat(0.0, category, "", ""),
            new Order_by_category());

    if(pos<0)
      return acc;

    s = (Stat) stats.get(pos);
    j = new Integer(s.get_name().substring(s.get_name().indexOf('_')+1)).intValue();
    k = Collections.binarySearch(times, new Times(j,0), new Order_by_index());
    acc.add(new Double(s.get_data()).doubleValue(), ((Times) times.get(k)).val);

    for(int i=pos+1; i<stats.size(); i++)
    {
      s = (Stat) stats.get(i);
      if(s.get_category().equals(category))
      {
        j = new Integer(s.get_name().substring(s.get_name().indexOf('_')+1)).intValue();
        k = Collections.binarySearch(times, new Times(j,0), new Order_by_index());
        acc.add(new Double(s.get_data()).doubleValue(), ((Times) times.get(k)).val);
      }
      else
        break;
    }
    for(int i=pos-1; i>=0; i--)
    {
      s = (Stat) stats.get(i);
      if(s.get_category().equals(category)) {
        j = new Integer(s.get_name().substring(s.get_name().indexOf('_')+1)).intValue();
        k = Collections.binarySearch(times, new Times(j,0), new Order_by_index());
        acc.add(new Double(s.get_data()).doubleValue(), ((Times) times.get(k)).val);
      }
      else
        break;
    }

    return acc;
  }

  public void body()
  {
    Stat stat;

    // Process Events until END_OF_SIMULATION is received
    Sim_event ev = new Sim_event();
    for( sim_get_next(ev);  ev.get_tag() != GridSimTags.END_OF_SIMULATION; sim_get_next(ev) )
    {
      if(!active)
        continue; // Skip processing of this event

      switch(ev.get_tag()) {
        case GridSimTags.RECORD_STATISTICS:
          if(ev.get_data() != null)
            RecordStat((Stat) ev.get_data());
          break;

        case GridSimTags.RETURN_ACC_STATISTICS_BY_CATEGORY:
            ReturnAccStatByCategory(ev);
          break;
      }
    }
//    System.out.println(get_name()+": terminated!");

    if(active) {
      outFile.close();
    }
  }

  public void RecordStat(Stat stat) {
    boolean flag = true;
    for(int i=0; i<exclude_from_processing.length; i++)
    {
      if((stat.get_category()+".").startsWith(exclude_from_processing[i]+"."))
      {
        flag = false;
        break;
      }
    }
    if(flag)
      stats.add(stat);

    flag = true;
    for(int i=0; i<exclude_from_file.length; i++)
    {
      if((stat.get_category()+".").startsWith(exclude_from_file[i]+"."))
      {
        flag = false;
        break;
      }
    }
    if(flag)
      outFile.println(stat.toString());
  }

  public void ReturnAccStatByCategory(Sim_event ev) {
    stats_sort_by_cd = new LinkedList(stats);
    Collections.sort(stats_sort_by_cd, new Order_by_category_data());
    Accumulator acc = accumulate((String) ev.get_data());
    Send(ev.get_src(), GridSimTags.SCHEDULE_NOW, ev.get_tag(), acc);
  }
}

