/**
 * Title:        GridSim Toolkit
 * Version:      GridSim  1.0, 12/09/2001
 * Description:  GridSim (Grid Simulation) Toolkit for Modeling and Simulation of
 *               Parallel and Distributed Systems such as Clusters and Grids
 * @author       R. Buyya and M. Murshed (by surname)
 *               Monash University, Melbourne, Australia
 * Licence:      GPL - http://www.gnu.org/copyleft/gpl.html
 */

package gridsim;
import java.util.*;
import java.io.*;
import eduni.simjava.*;

/**
 * A class representing a Machine of a Grid Resource.
 * It can contain one or more PEs.
 */
public class Machine
{
  private int id;
  // |PEs| > 1 is SMP
  private PEList PEs;
  private int SchedulingPolicy; // 0 = time shared; 1 = space shared

  public Machine(int id, PEList PEs, int SchedulingPolicy) {
    this.id = id;
    this.PEs = PEs;
    this.SchedulingPolicy = SchedulingPolicy;
  }

  public int GetID() {
    return id;
  }

  public int GetSize() { // number of PEs
    return PEs.size();
  }

  public PEList GetPEs() {
    return PEs;
  }

  public int GetMIPSRating() {
    int rating = 0;
    for(int i = 0; i < PEs.size(); i++) {
      rating += ((PE) PEs.get(i)).GetMIPSRating();
    }
    return rating;
  }

  public int ByteSize() {
    return 2*4 + PEs.ByteSize();
  }
}
